/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Finder.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import it.tidalwave.util.Finder.SortDirection;

/***********************************************************************************************************************
 *
 * A facility for performing queries on an {@link ObservationSet}. This class implements the
 * <a href="http://en.wikipedia.org/wiki/Fluent_interface">Fluent Interface</a> pattern. A typical usage is:
 *
 * <pre>
 * ObservationSet observationSet = ...;
 * List&lt;Observable&gt; results = observationSet.find(Observable.class).
 *                                           from(10).
 *                                           max(20).
 *                                           sort(SortCriterion.DISPLAY_NAME, SortDirection.ASCENDING).
 *                                           results();
 * </pre>
 * 
 * @author  Fabrizio Giudici
 * @version $Id: Finder.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
public interface Finder<T> extends it.tidalwave.util.Finder<T>
  {
    /***************************************************************************
     *
     * A set of criteria to sort results coming out of a query.
     *
     **************************************************************************/
    public static enum SortCriterion implements it.tidalwave.util.Finder.SortCriterion
      {
        /** Specifies to sort results by observation date. */
        DATE,
        /** Specifies to sort results by display name. */
        DISPLAY_NAME,
        /** Specifies to sort results by observable. */
        OBSERVABLE,
        /** Specifies to sort results by location. */
        LOCATION
      }

    /***************************************************************************
     *
     * Clones this {@code Finder}.
     *
     * TODO: what about having Finder immutable, and get rid of clone()?
     * 
     * @return  a cloned {@code Finder}
     *
     **************************************************************************/
    public Finder<T> clone();

    /***************************************************************************
     *
     * Configures the query so only results from the given position will be
     * returned.
     *
     * @param   firstResult  the index of the first item to return
     * @return               the {@code Finder}
     *
     **************************************************************************/
    @Nonnull
    @Override
    public Finder<T> from (@Nonnegative int firstResult);

    /***************************************************************************
     *
     * Configures the query so a maximum number of items will be returned.
     *
     * @param   maxResults   the max number of items that will be returned
     * @return               the {@code Finder}
     *
     **************************************************************************/
    @Nonnull
    @Override
    public Finder<T> max (@Nonnegative int maxResults);

    /***************************************************************************
     *
     * Configures the query with the given sort criteria.
     *
     * @param   criterion    the sort criterion
     * @param   direction    the sort direction
     * @return               the {@code Finder}
     *
     **************************************************************************/
    @Nonnull
    public Finder<T> sort (@Nonnull SortCriterion criterion, @Nonnull SortDirection direction);

    /***************************************************************************
     *
     * Configures the query with the given sort criteria.
     *
     * @param   criterion    the sort criterion
     * @return               the {@code Finder}
     *
     **************************************************************************/
    @Nonnull
    public Finder<T> sort (@Nonnull SortCriterion criterion);
  }
