/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Observation.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Date;
import java.io.Serializable;
import it.tidalwave.util.Initializer;
import javax.annotation.Nonnegative;

/*******************************************************************************
 *
 * Represents an observation, made at a certain time and {@link Location}, by
 * certain {@link Observer}s and composed of one or more {@link ObservationItem}s.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: Observation.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 ******************************************************************************/
public interface Observation extends Serializable
  {
    public static final Class<Observation> Observation = Observation.class;

    /***************************************************************************
     *
     * A builder used to create new {@link Observation}s. This class implements
     * the <a href="http://en.wikipedia.org/wiki/Fluent_interface">Fluent Interface</a>
     * pattern. A typical usage is:
     *
     * <pre>
     * ObservationSet observationSet = ...;
     * 
     * Date when = ...;
     * Location camargue = observationSet.findOrCreate(Location.class, "Camargue", ...);
     * Observer madeBy = observationSet.findOrCreate(Observer.class, "John Doe", ...);
     * Observable bird = observationSet.findOrCreate(Observable.class, "Flamingo", ...);
     * 
     * Observation observation = observationSet.createObservation().
     *                                          date(when).
     *                                          location(camargue).
     *                                          item(bird, Cardinality.create(4)).
     *                                          observer(madeBy).
     *                                          build();
     * </pre>
     *
     **************************************************************************/
    public static interface BaseBuilder
      {
        /***********************************************************************
         *
         * Sets the date of the {@link Observation} to create.
         *
         * @param  date  the date
         * @return       the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder date (@Nonnull Date date);

        /***********************************************************************
         *
         * Sets the {@code Location} of the {@link Observation} to create.
         *
         * @param  location   the {@code Location}
         * @return            the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder location (@Nonnull Location location);

        /***********************************************************************
         *
         * Adds an {@code Observable} to the {@link Observation} to create.
         *
         * @param  observable   the {@code Observable}
         * @param  cardinality  the {@code Cardinality}
         * @return              the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder item (@Nonnull Observable observable,
                             @Nonnull Cardinality cardinality);

        /***********************************************************************
         *
         * Adds an {@code Observable} to the {@link Observation} to create.
         *
         * @param  observable   the {@code Observable}
         * @param  cardinality  the {@code Cardinality}
         * @param  initializer  
         * @return              the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder item (@Nonnull Observable observable,
                             @Nonnull Cardinality count,
                             @Nonnull Initializer<ObservationItem> initializer);

        /***********************************************************************
         *
         * Sets the {@code Observer} of the {@link Observation} to create.
         *
         * @param  observer   the {@code Observer}
         * @return            the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder observer (@Nonnull Observer observer);

        /***********************************************************************
         *
         * Sets the {@code Source} of the {@link Observation} to create.
         *
         * @param  source     the {@code Source}
         * @return            the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder source (@Nonnull Source source);

        /***********************************************************************
         *
         * Sets the note of the {@link Observation} to create.
         *
         * @param  note       the note
         * @return            the {@code Builder}
         *
         **********************************************************************/
        @Nonnull
        public BaseBuilder note (@Nonnull String note);

        @Nonnegative
        public int getItemCount();
        
        /***********************************************************************
         *
         * Creates a new {@link Observation}.
         *
         * @return  the new {@code Observation}
         *
         **********************************************************************/
        @Nonnull
        public Observation build(); // FIXME move to BaseBuilder

        @Nonnull
        public Observation apply(); // FIXME move to ReplaceBuilder
      }

    public static interface Builder extends BaseBuilder
      {
        // TODO: needs to override all the methods to return a Builder
      }

    public static interface ReplaceBuilder extends BaseBuilder
      {
        @Nonnull
        public ReplaceBuilder removeItem (@Nonnull ObservationItem observationItem);
        // TODO: needs to override all the methods to return a ReplaceBuilder
      }

    /***************************************************************************
     *
     * Returns the time at which this {@code Observation} has been made.
     *
     * @return  the time
     *
     **************************************************************************/
    @CheckForNull
    public Date getDate();

    /***************************************************************************
     *
     * Returns the {@link Location} where this {@code Observation} has been made.
     *
     * @return  the {@code Location}
     *
     **************************************************************************/
    @Nonnull
    public Location getLocation();

    /***************************************************************************
     *
     * Returns a {@link Finder} for {@link ObservationItem}s that are part of
     * this {@code Observation}.
     *
     * FIXME: use a single find(Class)
     * @return  the {@link Finder}
     *
     **************************************************************************/
    @Nonnull
    public Finder<ObservationItem> findObservationItems();

    /***************************************************************************
     *
     * Returns a {@link Finder} for {@link Observer}s that made
     * this {@code Observation}.
     * FIXME: use a single find(Class)
     *
     * @return  the {@link Finder}
     *
     **************************************************************************/
    @Nonnull
    public Finder<Observer> findObservers();

    /***************************************************************************
     *
     * Creates a Builder that allows to modify this observation and replace the original.
     *
     **************************************************************************/
    public ReplaceBuilder replace();
  }
